class NinjaPlugin:
    def __init__(self, engine):
        pass

    def on_translated(self, old, new):
        # Do nothing by default
        pass

    def on_stroked(self, stroke):
        # Do nothing by default
        pass
